(load "~/quicklisp/setup.lisp")
(ql:quickload '(hunchentoot clack dexador com.inuoe.jzon serapeum))

(uiop:define-package tfc-server
  (:use :cl :uiop :com.inuoe.jzon :clack))

(in-package :tfc-server)

(defvar *last*)
(defvar *last-data*)
(defvar *last-list-data*)
(defvar *stream*)
(defvar *token* "boFFRM68vvXbO-DO7S9YDg86lHX027-hd07mn0dh")

(setf hunchentoot:*default-connection-timeout* 240)

(defvar *server* (hunchentoot:start (make-instance 'hunchentoot:easy-acceptor :port 4242)))

(hunchentoot:define-easy-handler (say-yo :uri "/yo") (name)
  (setf (hunchentoot:content-type*) "text/plain")
  (format nil "Hey~@[ ~A~]!" name))

(hunchentoot:define-easy-handler (respond :uri "/health-form") ()
  (setf (hunchentoot:content-type*) "application/json")
  (let ((request-type (hunchentoot:request-method hunchentoot:*request*)))
    (cond ((eq request-type :get) nil)
          ((eq request-type :post)
           (uiop:println hunchentoot:*request*)
           (setq *last* hunchentoot:*request*)
           (setq *stream* (hunchentoot::content-stream hunchentoot:*request*))
           (let* ((stream (hunchentoot::content-stream hunchentoot:*request*))
                  (data (hunchentoot::get-post-data :request *last*))
                  (content-type (hunchentoot:header-in "content-type" hunchentoot:*request*))
                  (boundary (let ((position (position #\= content-type)))
                              (when position
                                (subseq content-type (1+ position)))))
                  (string-data (flexi-streams:octets-to-string data))
                  (parts (ppcre:split boundary string-data)))

             (setq *last-data* parts)
             (setq *last-list-data* nil)
             (loop :for i :in *last-data*
                   :do (let* ((content-disposition (nth 1
                                            (serapeum:lines i :eol-style :crlf :honor-crlf t)))
                              (start (if content-disposition (position #\" content-disposition)))
                              (end (if start (position #\" content-disposition :start (1+ start)) nil))
                              (name (if end (if start (subseq content-disposition (1+ start) end) nil) nil))
                              (content (if (string= name "image")
                                           (rest
                                            (rest
                                             (rest
                                              (rest
                                               (serapeum:lines i :eol-style :crlf
                                                                 :honor-crlf t :keep-eols t)))))
                                           (nth 3 (serapeum:lines i :eol-style :crlf
                                                                    :honor-crlf t))))
                              (file-start (if (string= name "image")
                                              (position #\" content-disposition :start 44)))
                              (file-end (if file-start
                                            (position #\" content-disposition :start (1+ file-start)) nil))
                              (image-file-name (if (string= name "image")
                                                   (subseq content-disposition
                                                           (1+ file-start) file-end) nil))
                              (form-list '()))

                         ;; This gets rid of beginning and ending NIL values
                         (if name (if (string/= name "image") (progn
                                                                (format t "~&~A: ~A" name content)
                                                                (setq form-list (list (cons name content))))) nil)
                         (if (string= name "image") (setq content (butlast content)))

                         (if (string= "image" name)
                             (progn (format t "~&~A: ~A" name image-file-name)
                                    (setq form-list (list name image-file-name))
                                    (save-string-file
                                     (apply #'concatenate 'string content) image-file-name)))

                         (setq *last-list-data* (append *last-list-data* form-list)))))))))


(defun display-stream (stream)
  (loop for line = (read-line stream nil)
        while line do (format t "~A~%" line)))

(defun print-hash-entry (key value)
  (format t "~S: ~S~%" key (if (hash-table-p value) (maphash 'print-hash-entry value) value)))

(defun process-form (form)
  "Processes the form and sends it on"
    (uiop:println "---")
    (maphash 'print-hash-entry form)
    (uiop:println "---"))


(defun render-json (object)
  (uiop:println (format nil "Json: ~A" (com.inuoe.jzon:stringify object))))

(defun save-string-file (string file-path)
  "save a file that is represented as a string to disk that came from a multipart/form-data"
  (serapeum/bundle:write-string-into-file string file-path
                                          :if-exists :overwrite
                                          :if-does-not-exist :create
                                          :external-format :latin-1))

